@@ -38,7 +38,7 @@
 #endif
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.182 2011/03/27 18:50:04 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/funcs.c,v 1.186 2011/05/06 15:41:23 tg Exp $");
 
 #if HAVE_KILLPG
 /*
@@ -205,7 +205,7 @@ static void ptest_error(Test_env *, int, const char *);
 static char *kill_fmt_entry(char *, int, int, const void *);
 static void p_time(struct shf *, bool, long, int, int,
     const char *, const char *)
-    MKSH_A_NONNULL((nonnull (6, 7)));
+    MKSH_A_NONNULL((__nonnull__ (6, 7)));
 
 int
 c_pwd(const char **wp)
@@ -794,7 +794,8 @@ c_typeset(const char **wp)
 					f->flag |= fset;
 					f->flag &= ~fclr;
 				} else {
-					fpFUNCTf(shl_stdout, 0, f->flag & FKSH,
+					fpFUNCTf(shl_stdout, 0,
+					    tobool(f->flag & FKSH),
 					    wp[i], f->val.t);
 					shf_putc('\n', shl_stdout);
 				}
@@ -819,7 +820,7 @@ c_typeset(const char **wp)
 					continue;
 				if (thing == '-')
 					fpFUNCTf(shl_stdout, 0,
-					    vp->flag & FKSH,
+					    tobool(vp->flag & FKSH),
 					    vp->name, vp->val.t);
 				else
 					shf_puts(vp->name, shl_stdout);
@@ -2541,11 +2542,23 @@ c_test(const char **wp)
 	 * our parser does the right thing for the omitted steps.
 	 */
 	if (argc <= 5) {
-		const char **owp = wp;
+		const char **owp = wp, **owpend = te.wp_end;
 		int invert = 0;
 		Test_op op;
 		const char *opnd1, *opnd2;
 
+		if (argc >= 2 && ((*te.isa)(&te, TM_OPAREN))) {
+			te.pos.wp = te.wp_end - 1;
+			if ((*te.isa)(&te, TM_CPAREN)) {
+				argc -= 2;
+				te.wp_end--;
+				te.pos.wp = owp + 2;
+			} else {
+				te.pos.wp = owp + 1;
+				te.wp_end = owpend;
+			}
+		}
+
 		while (--argc >= 0) {
 			if ((*te.isa)(&te, TM_END))
 				return (!0);
@@ -2566,8 +2579,6 @@ c_test(const char **wp)
 			}
 			if (argc == 1) {
 				opnd1 = (*te.getopnd)(&te, TO_NONOP, 1);
-				if (strcmp(opnd1, "-t") == 0)
-				    break;
 				res = (*te.eval)(&te, TO_STNZE, opnd1,
 				    NULL, 1);
 				if (invert & 1)
@@ -2580,6 +2591,7 @@ c_test(const char **wp)
 				break;
 		}
 		te.pos.wp = owp + 1;
+		te.wp_end = owpend;
 	}
 
 	return (test_parse(&te));
