@@ -33,7 +33,7 @@
 #include <locale.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.187 2011/03/27 18:50:05 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.190 2011/04/17 12:24:43 tg Exp $");
 
 extern char **environ;
 
@@ -112,6 +112,10 @@ rndsetup(void)
 	char *cp;
 
 	cp = alloc(sizeof(*bufptr) - ALLOC_SIZE, APERM);
+#ifdef DEBUG
+	/* clear the allocated space, for valgrind */
+	memset(cp, 0, sizeof(*bufptr) - ALLOC_SIZE);
+#endif
 	/* undo what alloc() did to the malloc result address */
 	bufptr = (void *)(cp - ALLOC_SIZE);
 	/* PIE or something similar provides us with deltas here */
@@ -546,13 +550,14 @@ main(int argc, const char *argv[])
 	}
 	Flag(FERREXIT) = errexit;
 
-	if (Flag(FTALKING)) {
+	if (Flag(FTALKING))
 		hist_init(s);
-		alarm_init();
-	} else
+	else
 		/* set after ENV */
 		Flag(FTRACKALL) = 1;
 
+	alarm_init();
+
 	if (Flag(FAS_BUILTIN))
 		return (shcomexec(l->argv));
 
@@ -1005,7 +1010,7 @@ tty_close(void)
 #define VWARNINGF_BUILTIN	4
 #define VWARNINGF_INTERNAL	8
 
-static void MKSH_A_FORMAT(printf, 2, 0)
+static void MKSH_A_FORMAT(__printf__, 2, 0)
 vwarningf(unsigned int flags, const char *fmt, va_list ap)
 {
 	if (*fmt != 1) {
