@@ -22,7 +22,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.59 2011/03/26 15:32:37 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.64 2011/05/07 00:51:12 tg Exp $");
 
 extern short subshell_nesting_level;
 
@@ -166,7 +166,7 @@ static struct ioword *
 synio(int cf)
 {
 	struct ioword *iop;
-	static struct ioword *nextiop = NULL;
+	static struct ioword *nextiop;
 	bool ishere;
 
 	if (nextiop != NULL) {
@@ -179,12 +179,15 @@ synio(int cf)
 		return (NULL);
 	ACCEPT;
 	iop = yylval.iop;
-	ishere = (iop->flag&IOTYPE) == IOHERE;
+	if (iop->flag & IONDELIM)
+		goto gotnulldelim;
+	ishere = (iop->flag & IOTYPE) == IOHERE;
 	musthave(LWORD, ishere ? HEREDELIM : 0);
 	if (ishere) {
 		iop->delim = yylval.cp;
 		if (*ident != 0)
 			/* unquoted */
+ gotnulldelim:
 			iop->flag |= IOEVAL;
 		if (herep > &heres[HERES - 1])
 			yyerror("too many %ss\n", "<<");
@@ -629,7 +632,7 @@ function_body(char *name,
 	struct op *t;
 	bool old_func_parse;
 
-	sname = wdstrip(name, false, false);
+	sname = wdstrip(name, 0);
 	/*-
 	 * Check for valid characters in name. POSIX and AT&T ksh93 say
 	 * only allow [a-zA-Z_0-9] but this allows more as old pdkshs
@@ -649,15 +652,11 @@ function_body(char *name,
 	 */
 	if (ksh_func) {
 		if (tpeek(CONTIN|KEYWORD|ALIAS) == '(' /*)*/) {
-			struct tbl *tp;
-
 			/* function foo () { */
 			ACCEPT;
 			musthave(')', 0);
 			/* degrade to POSIX function */
 			ksh_func = false;
-			if ((tp = ktsearch(&aliases, sname, hash(sname))))
-				ktdelete(tp);
 		}
 		musthave('{' /*}*/, CONTIN|KEYWORD|ALIAS);
 		REJECT;
@@ -665,7 +664,7 @@ function_body(char *name,
 
 	t = newtp(TFUNCT);
 	t->str = sname;
-	t->u.ksh_func = ksh_func;
+	t->u.ksh_func = tobool(ksh_func);
 	t->lineno = source->line;
 
 	old_func_parse = e->flags & EF_FUNC_PARSE;
